#include <Arduino.h>
#include "XC4607.h"

//
// Hardware connections:
//
// Port B 4..0 = LAT, D, C, B, A
// Port D 7..6 = DI, CLK
//

XC4607::XC4607() {
  DDRB |= 0x1f; // Port B 4..0 are outputs
  DDRD |= 0xc0; // Port D 7..6 are outputs
  vsync = false;
}

void XC4607::wait_vsync() {
  while(!vsync);
  vsync = false;
}


void XC4607::tick() {
  int data = ~buffer[row ^ 0xf];
  int val;
  for (int i = 0; i <= 15; i++) {
    // Write data bit to DI, CLK low
    int val = (PORTD & 0x3f) | ((data & 0x8000) >> 8);
    PORTD = val;
    // CLK high
    val |= 0x40;
    PORTD = val;
    data <<= 1;
  }
  // Write row number, LAT high
  val = (PORTB & 0xe0) | row;
  PORTB = (val | 0x10);
  // LAT low
  PORTB = val;
  if (row == 15) {
    row = 0;
    vsync = true;
  }
  else
    ++row;
}

void XC4607::cls() {
  for (int i = 0; i <= 15; i++)
    buffer[i] = 0;
}

void XC4607::set(int x, int y) {
  buffer[y] |= 0x8000 >> x;
}

void XC4607::clr(int x, int y) {
  buffer[y] &= ~(0x8000 >> x);
}

void XC4607::hline(int x, int y, int length) {
  while (length--)
    set(x++, y);
}

void XC4607::vline(int x, int y, int length) {
  while (length--)
    set(x, y++);
}

void XC4607::put(int x, int y, unsigned int data, int length) {
  int mask = (~(0xffff >> length) >> x);
  buffer[y] = (buffer[y] & ~mask) | (data << (16 - length - x));
}

void XC4607::blit8(int x, int y, byte *data, int width, int height, int stride = 1) {
  while (height--) {
    put(x, y++, *data, width);
    data += stride;
  }
}

void XC4607::blit16(int x, int y, unsigned int *data, int width, int height, int stride = 1) {
  while (height--) {
    put(x, y++, *data, width);
    data += stride;
  }
}

void XC4607::write(int x, int y, Font *font, char *text) {
  char c;
  while (c = *text++) {
    char *p = font->characters;
    while (*p && *p != c)
      ++p;
    if (*p) {
      int i = p - font->characters;
      blit8(x, y, font->data + i, font->char_width, font->char_height, font->data_width);
      x += font->char_width + 1;
    }
  }
}
